//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//  
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//  
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file
// COPYRIGHT, located in the top directory of this software.
//
//
// ************************************************************************
//
// $Source: /togcvs/adl2/src/examples/idl/stock_example/StockBrokerImpl.java,v $
// $Revision: 1.3 $
// $Date: 1998/02/27 15:46:35 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	stock broker example
// NAME:	StockBrokerImpl.java
// 	
// HISTORY:
// 	Cyril Inard d'Argence, The Open Group - Jan 13, 1998: Created
// 		
// ************************************************************************

package stock_example;
import  stock_example.stock_app.*;

import java.util.*;

/**
 * Implementation of the StockBroker CORBA object.
 */

public class StockBrokerImpl extends _StockBrokerImplBase {

  private int myAccount         = 0;
  private int nbProducts        = 0;
  private final int maxProducts = 100;
  private Hashtable theProducts = new Hashtable(maxProducts);

  public void newProduct(String name, int nb_of_shares) throws BadCallExc
  {
    int price = 0;
    try
    {
      price = ProductsBank.get(name);
    }
    catch(UnknownProductExc e)
    { 
       /* not dealt with here, to simplify */ 
       throw (new BadCallExc(true,false));
    }
    Product p = new Product(name, nb_of_shares);
    theProducts.put(name, p);
    nbProducts++;
    myAccount += price * nb_of_shares;
  }
  
  public int getPriceSymbol(String symbol) throws BadCallExc
  {
     int ret;
     Product p = (Product)theProducts.get(symbol);
     if (p == null)
     {
        throw (new BadCallExc(true,false));
     }
     else
     {
        try 
        {
           ret = p.getPrice();
        }
        catch(UnknownProductExc e)
        {
           throw (new BadCallExc(true,false));
        }
        return ret;
     }
  }


  public int Cash_Balance() 
  {
     return myAccount;
  }

  public int Stock_Balance(String symbol) throws BadCallExc
  {
    Product p = (Product)theProducts.get(symbol);
    if (p == null)
      throw (new BadCallExc(true,false));
    else
      return p.getStock();
  }

  public void Buy(String symbol, int no_of_shares) throws BadCallExc
  {
     Product p = (Product)theProducts.get(symbol);
     if (p == null)
     {
        throw (new BadCallExc(true,false));
     }
     else 
     {
        p.add(no_of_shares);
        myAccount -= getPriceSymbol(symbol) * no_of_shares;
     }
  }

  public void Sell(String symbol, int no_of_shares) throws BadCallExc
  {
     Product p = (Product)theProducts.get(symbol);
     if (p == null)
     {
        throw (new BadCallExc(true,false));
     }
     else
     {
        try
        {
           p.remove(no_of_shares);
           myAccount += getPriceSymbol(symbol) * no_of_shares;
        }
        catch(InsufficientStockExc e)
        {
           throw (new BadCallExc(false,true));
        }
     }
  }
}
// ***********************************************************************
// History Log:
// $Log: StockBrokerImpl.java,v $
// Revision 1.3  1998/02/27 15:46:35  cinard
// Updated copyright notice.
//
// Revision 1.2  1998/02/12 16:04:22  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/10 17:36:09  cinard
// Implementations for IDL interfaces and other useful classes.
//
// ***********************************************************************
