// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ************************************************************************
//
// $Source: /togcvs/adl2/src/examples/java/stock_example/StockBroker.java,v $
// $Revision: 1.2 $
// $Date: 1998/02/27 10:49:05 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	stock example
// NAME:	StockBroker.java
// 	
// HISTORY:
// 	Cyrille Bareau, The Open Group RI - Nov 1997: Created
// 		
// ************************************************************************

package stock_example;

import java.util.*;


public class StockBroker {

  static final public int BAD_SYMBOL = 0;
  static final public int INSUF_STOCK = 1;

  private int myAccount;
  private int nbProducts;
  private final int maxProducts = 100;
  private Hashtable theProducts;

  public StockBroker() {
    myAccount = 0;
    theProducts = new Hashtable(maxProducts);
    nbProducts = 0;
  }

  public void newProduct(String name, int nb_of_shares) throws BadCallExc {
    int price = 0;
    try {
      price = ProductsBank.get(name);
    } catch(UnknownProductExc e) { /* not dealt with here, to simplify */ 
      throw (new BadCallExc(BAD_SYMBOL));
    }
    Product p = new Product(name, nb_of_shares);
    theProducts.put(name, p);
    nbProducts++;
    myAccount += price * nb_of_shares;
  }

  /*
  public void addProduct(String s, int nb_shares) throws UnknownProductExc {
    Product p = (Product)theProducts.get(s);
    if (p == null)
      throw (new UnknownProductExc());
    else
      p.add(nb_shares);
  }
  */

  public int getPriceSymbol(String symbol) throws BadCallExc {
    int ret;
    Product p = (Product)theProducts.get(symbol);
    if (p == null)
      throw (new BadCallExc(BAD_SYMBOL));
    else {
      try {
        ret = p.getPrice();
      } catch(UnknownProductExc e) {
        throw (new BadCallExc(BAD_SYMBOL));
      }
      return ret;
    }
  }


  public int Cash_balance() {
    return myAccount;
  }

  public int Stock_balance(String symbol) throws BadCallExc {
    Product p = (Product)theProducts.get(symbol);
    if (p == null)
      throw (new BadCallExc(BAD_SYMBOL));
    else
      return p.getStock();
  }

  public void Buy(String symbol, int no_of_shares) 
    throws BadCallExc {
    Product p = (Product)theProducts.get(symbol);
    if (p == null)
      throw (new BadCallExc(BAD_SYMBOL));
    else {
      p.add(no_of_shares);
      myAccount -= getPriceSymbol(symbol) * no_of_shares;
    }
  }

  public void Sell(String symbol, int no_of_shares) 
    throws BadCallExc {
    Product p = (Product)theProducts.get(symbol);
    if (p == null)
      throw (new BadCallExc(BAD_SYMBOL));
    else {
      try {
        p.remove(no_of_shares);
        myAccount += getPriceSymbol(symbol) * no_of_shares;
      } catch(InsufficientStockExc e) {
        throw (new BadCallExc(INSUF_STOCK));
      }
    }
  }
}
