//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/frontend/adlt.c,v $
// $Revision: 1.2 $
// $Date: 1998/08/06 14:08:44 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADLT C driver
// NAME:        adlt.c
//
// HISTORY:
//      Lionel Mallet, The Open Group RI - Jan 27, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#include <sys/wait.h>
#endif
#include <ctype.h>

#ifdef WIN32
    char dirsep='\\';
    char pathsep=';';
#define putenv _putenv
#else
    char dirsep='/';
    char pathsep=':';
#endif

#define ADLT_SYSTEMFAILED 200
static char *def_adlhome = ADLHOME;

int main(int argc, char *argv[], char *envp[]) {
  char *adlhomeptr = NULL;
  char adlhome[2048];
  char *classpathptr = NULL;
  char classpathbuf[2048];
  char cmdbuf[4096];
  char optbuf[1024];
  char **javaenv;
  int numenv = 0;
  int i;
  int exitStatus;

  /* get ADL2HOME */
  adlhomeptr = getenv("ADL2HOME");
  if (adlhomeptr == NULL) {
    adlhomeptr =  def_adlhome;
  }

  /* extend CLASSPATH with ADL2HOME/lib/adlt.jar and JGLHOME */
  classpathptr = getenv("CLASSPATH");

  if (classpathptr == NULL) {
    sprintf(classpathbuf, 
	    "CLASSPATH=%s%clib%cadlt.jar%c%s",
	    adlhomeptr, dirsep, dirsep,
	    pathsep, JGLHOME);
  } else {
    sprintf(classpathbuf,
	    "CLASSPATH=%s%c%s%clib%cadlt.jar%c%s",
	    classpathptr, pathsep,
	    adlhomeptr, dirsep, dirsep,
	    pathsep, JGLHOME);
  }
  
#ifdef DEBUG
  /* extend CLASSPATH with ADL2HOME/build */
  sprintf(classpathbuf, "%s%c%s%cbuild", 
	  classpathbuf, pathsep, adlhomeptr, dirsep);
  printf("%s\n", classpathbuf);
#endif
  putenv(classpathbuf);

  /* initialize java command */
  sprintf(cmdbuf, "%s -Dadl.home=%s", JAVACMD, adlhomeptr );

  /* search for environment variables that start with ADL_ and convert
     them as java properties */
  i = 0;
  while (envp[i] != NULL) {
    if (strncmp(envp[i], "ADL_", 4) == 0) {
      /* matches ADL_* */
      int quoteRequired = 0;
      int j = 0;
      char *c = &envp[i][4];

      /* copy variable name in lowercase */
      while (*c != '=') {
	optbuf[j++] = tolower(*c++);
      }

      /* copy the = char and add a " */
      optbuf[j++] = *c++;
      if (*c != '"' && *c != '\'') {
        quoteRequired = 1;
        optbuf[j++] = '"';
      }

      /* now copy value */
      while (*c != 0) {
	optbuf[j++] = *c++;
      }

      /* final " and \0 */
      if (quoteRequired) optbuf[j++] = '"';
      optbuf[j] = 0;

#ifdef DEBUG
      printf("Added property adl.%s\n", optbuf);
#endif

      /* extend java command */
      sprintf(cmdbuf, "%s -Dadl.%s", cmdbuf, optbuf);
    }
    i++;
  }

  /* now append main class of application and command line arguments
     to java command */
  sprintf(cmdbuf, "%s %s", cmdbuf, ADLTCLASS);
  for (i = 1; i < argc; i++) {
    sprintf(cmdbuf, "%s %s", cmdbuf, argv[i]);
  }
  
#ifdef DEBUG
  printf("%s\n", cmdbuf);
#endif

  exitStatus = system(cmdbuf);
#ifdef WIN32
  return exitStatus?exitStatus:ADLT_SYSTEMFAILED;
#else
  return (WIFEXITED(exitStatus) ?
	  WEXITSTATUS(exitStatus) :
	  ADLT_SYSTEMFAILED);
#endif

}

// ***********************************************************************
// History Log:
// $Log: adlt.c,v $
// Revision 1.2  1998/08/06 14:08:44  mallet
// Merged Shane's last change in the v1_2-patches branch back onto the trunk.
//
// Revision 1.1.2.1  1998/08/05 00:33:01  shane
// Moved adlt.c to its own tree for building distributions.
//
// Revision 1.8  1998/06/05 15:45:43  shane
// Added some code to fix compilation errors on Windows.
//
// Revision 1.7  1998/05/15 14:04:37  mallet
// Added required header file <sys/wait.h>: to be checked for NT.
//
// Revision 1.6  1998/05/13 14:08:54  mallet
// Fixed handling of return status from system(): when system failed, an
// ADLT_SYSTEMFAILED (200) status is returned.
//
// Revision 1.5  1998/02/27 13:49:26  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 14:55:10  bareau
// Bug fixed: quotes were added even when not required (value already surrounded by quotes).
//
// Revision 1.3  1998/02/09 15:40:26  mallet
// Fixed bug when value of an ADL_* environment variable is made of
// several words separated by spaces: now systematically add " around the
// value.
//
// Revision 1.2  1998/01/28 16:49:49  mallet
// Removed . from list of directories added to the class path.
// $ADL2HOME/build is now added only when DEBUG is defined during
// compilation.
//
// Revision 1.1  1998/01/27 18:16:07  mallet
// C driver for the ADL 2.0 Translation System.
//
// ***********************************************************************
