//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/c/ADLDataSet.h,v $
// $Revision: 1.4 $
// $Date: 1998/07/21 13:45:05 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment - TDD datasets root routines
// NAME:	ADLDataSet.h
// 	
// HISTORY:
//      Shane McCarron, The Open Group - April 10, 1998: Converted to C
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLDataSet_h
#define ADLDataSet_h

#include "ADLRuntimeException.h"
#include <stdlib.h>

typedef enum { 
    charT, 
    unsigned_charT, 
    shortT, 
    unsigned_shortT, 
    intT, 
    unsigned_intT, 
    longT, 
    unsigned_longT, 
    floatT, 
    doubleT, 
    long_doubleT,
    voidT
} ADL_LiteralType;


struct ADLDataSet {
    char *label;
    int numValues;
    struct ADLDataSet *nextSet;
    void *(*provide)(struct ADLDataSet *, int);
    int (*relinquish)(struct ADLDataSet *, int); 

    union {
    // Fields for ConcatentationSet
	struct c {
	    int numElements;
	    struct ADLDataSet *elementSet;
	    struct ADLDataSet *providedSetPtr;
	    int providedSet;
	    int providedIndex;
	} c;
    // Fields for FactorySet
    	struct f {
	    char *factoryName;
	    struct ADLParameterSet *params;
	    void **factoryParameters;
	    void *providedObject;
	    void *(*factoryProvide)(void **);
	    int (*factoryRelinquish)(void **, void *);
	} f;
    // Fields for LiteralSet
	struct l {
	    ADL_LiteralType type;
	    short typeSize;
	    void *values;
	    short chunkNumberOfValues;
	    short maxNumberOfValues;
	} l;
    } mode;

};

typedef struct ADLDataSet ADLDataSet;

typedef struct ADLDataSet ADLFactorySet;
typedef struct ADLDataSet ADLLiteralSet;
typedef struct ADLDataSet ADLConcatenationSet;


#endif // ADLDataSet_h

// ***********************************************************************
// History Log:
// $Log: ADLDataSet.h,v $
// Revision 1.4  1998/07/21 13:45:05  mallet
// Fixed typo in "concatenation".
//
// Revision 1.3  1998/05/08 20:21:06  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.2  1998/04/29 15:58:39  shane
// Fixed my stupid unions that didn't work right.
//
// Revision 1.1  1998/04/16 12:09:09  shane
// First checkin of complete set.
//
//
// ***********************************************************************
