//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/c/ADLLiteralSet.c,v $
// $Revision: 1.5 $
// $Date: 1998/07/21 13:37:11 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment - Test directive and factory 
//                                      parameter set class
// NAME:	ADLLiteralSet.c
// 	
// HISTORY:
// 	Shane McCarron, The Open Group - 13 April 1998: Converted
//      Lionel Mallet, The Open Group RI  - Oct  28, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLLiteralSet.h"
#include "ADLMemory.h"

static short typeSizes[] = {
    sizeof(char), 
    sizeof(unsigned char), 
    sizeof(short), 
    sizeof(unsigned short), 
    sizeof(int),
    sizeof(unsigned int),
    sizeof(long),
    sizeof(unsigned long),
    sizeof(float),
    sizeof(double),
    sizeof(long double),
    sizeof(void *)
    };

static void *provide(ADLDataSet *d, int index) {

  void *eltAdr;

  eltAdr = (void *)((char *)d->mode.l.values + (index * d->mode.l.typeSize));
  return eltAdr;
}

static int relinquish(ADLDataSet *d, int index) {
  return 1;
}

ADLDataSet *ADL_LiteralSetInit(char *label, ADL_LiteralType type, void *vals, int size) {
  ADLDataSet *set;

  set = ADL__calloc(1, sizeof(ADLDataSet));
  if (set) {
      set->label = ADL__new_string(label);
      set->mode.l.type = type;
      set->mode.l.typeSize = typeSizes[type];
      set->provide = &provide;
      set->relinquish = &relinquish;
      set->mode.l.chunkNumberOfValues = ADL_chunkNumberOfValues;
      set->mode.l.maxNumberOfValues = 0;
      set->mode.l.values = NULL;
      set->numValues = 0;
      if (size) {
	  set->mode.l.maxNumberOfValues = size;

	  set->numValues = size;
	  set->mode.l.values = ADL__calloc(set->numValues, set->mode.l.typeSize);
	  if (set->mode.l.values) {
	      memcpy((char *)set->mode.l.values, vals, set->numValues * set->mode.l.typeSize);
	  } else {
	      ADL__free(set);
	      set = NULL;
	  }
      }
  }
  return set;
}

static void doAddValue(ADLDataSet *d, void *value) {
  if (d->numValues == 0) {
    // needs malloc
    d->mode.l.maxNumberOfValues = d->mode.l.chunkNumberOfValues;
    d->mode.l.values = ADL__calloc(d->mode.l.maxNumberOfValues, d->mode.l.typeSize);
  } else if (d->numValues == d->mode.l.maxNumberOfValues) {
    // needs realloc
    d->mode.l.maxNumberOfValues += d->mode.l.chunkNumberOfValues;
    d->mode.l.values = ADL__realloc(d->mode.l.values, d->mode.l.maxNumberOfValues * d->mode.l.typeSize);
  }

  if (d->mode.l.values == NULL) {
    // malloc/realloc failed
    ADL_RaiseException(ADLRuntime, 
			      "ADLLiteralSet: can't malloc/realloc to add new value!");
  } else {
    memcpy(((char *)d->mode.l.values + (d->numValues * d->mode.l.typeSize)), value, d->mode.l.typeSize);
    d->numValues++;
  }
}

void ADL_LiteralAddValue(ADLDataSet *d, void *value) {
  doAddValue(d, value);
}

void ADL_LiteralAddRangeValues(ADLDataSet *d, long minValue, long maxValue) {

  char c;
  unsigned char uc;
  short s;
  unsigned short us;
  int i;
  unsigned int ui;
  long l;
  unsigned long ul;

  switch (d->mode.l.type) {
      case charT:
	if ((char)minValue < (char)maxValue) {
	  for (c = (char)minValue; c <= (char)maxValue; c++) {
	    doAddValue(d, &c);
	  }
	} else {
	  for (c = (char)minValue; c >= (char)maxValue; c--) {
	    doAddValue(d, &c);
	  }
	}
	break;
      case unsigned_charT:
	if ((unsigned char)minValue < (unsigned char)maxValue) {
	  for (uc = minValue; uc <= maxValue; uc++) {
	    doAddValue(d, &uc);
	  }
	} else {
	  for (uc = (unsigned char)minValue; uc >= (unsigned char)maxValue; uc--) {
	    doAddValue(d, &uc);
	  }
	}
	break;
      case shortT:
	if ((short) minValue < (short) maxValue) {
	  for (s = (short)minValue; s <= (short)maxValue; s++) {
	    doAddValue(d, &s);
	  }
	} else {
	  for (s = (short)minValue; s >= (short)maxValue; s--) {
	    doAddValue(d, &s);
	  }
	}
	break;
      case unsigned_shortT:
	if ((unsigned short)minValue < (unsigned short)maxValue) {
	  for (us = (unsigned short)minValue; us <= (unsigned short)maxValue; 
		  us++) {
	    doAddValue(d, &us);
	  }
	} else {
	  for (us = (unsigned short) minValue; us >= (unsigned short) maxValue;
		  us--) {
	    doAddValue(d, &us);
	  }
	}
	break;
    case intT:
	if ((int)minValue < (int)maxValue) {
	  for (i = (int) minValue; i <= (int) maxValue; i++) {
	    doAddValue(d, &i);
	  }
	} else {
	  for (i = (int) minValue; i >= (int) maxValue; i--) {
	    doAddValue(d, &i);
	  }
	}
	break;
    case unsigned_intT:
	if ((unsigned int)minValue < (unsigned int)maxValue) {
	  for (ui = (unsigned int) minValue; ui <= (unsigned int) maxValue; ui++) {
	    doAddValue(d, &ui);
	  }
	} else {
	  for (ui = (unsigned int) minValue; ui >= (unsigned int) maxValue; ui--) {
	    doAddValue(d, &ui);
	  }
	}
	break;
    case longT:
	if ((long)minValue < (long)maxValue) {
	  for (l = (long)minValue; l <= (long)maxValue; l++) {
	    doAddValue(d, &l);
	  }
	} else {
	  for (l = (long)minValue; l >= (long)maxValue; l--) {
	    doAddValue(d, &l);
	  }
	}
	break;
    case unsigned_longT:
	if ((unsigned long)minValue < (unsigned long)maxValue) {
	  for (ul = (unsigned long)minValue; ul <= (unsigned long)maxValue; 
		  ul++) {
	    doAddValue(d, &ul);
	  }
	} else {
	  for (ul = (unsigned long)minValue; ul >= (unsigned long)maxValue; 
		  ul--) {
	    doAddValue(d, &ul);
	  }
	}
	break;
    }
}

// ***********************************************************************
// History Log:
//
// $Log: ADLLiteralSet.c,v $
// Revision 1.5  1998/07/21 13:37:11  mallet
// Changed label initialization to use ADL__new_string().
//
// Revision 1.4  1998/05/11 12:54:08  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.3  1998/05/08 20:21:06  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.2  1998/04/29 15:58:39  shane
// Fixed my stupid unions that didn't work right.
//
//
// ***********************************************************************
