//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/c/ADLMemory.c,v $
// $Revision: 1.3 $
// $Date: 1998/07/21 13:35:41 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment
// NAME:	ADLMemory.c
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - April 10 1998
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLMemory.h"
#include "stdlib.h"
#include "ADLRuntimeException.h"

/*
 * ADL__malloc(size_t bytes) - allocate memory and return a pointer to it.
 *
 * Fails out gracefully if memory cannot be allocated.
 */

void *ADL__malloc(size_t bytes) {
    void *ptr = malloc(bytes);
    if (ptr == NULL) {
	ADL_RaiseException(ADLOutOfMemory, "Memory allocation of %ld bytes failed", bytes);
	return(NULL);
    } else {
	return ptr;
    }
}

/*
 * ADL__calloc(size_t num, size_t size) - allocate memory and return a pointer to it.
 *
 * Fails out gracefully if memory cannot be allocated.
 */

void *ADL__calloc(size_t size, size_t bytes) {
    void *ptr = calloc(size, bytes);
    if (ptr == NULL) {
	ADL_RaiseException(ADLOutOfMemory, "Memory allocation of %ld elements of %ld bytes failed", size, bytes);
	return(NULL);
    } else {
	return ptr;
    }
}

/*
 * ADL__realloc(void *, size_t size) - allocate memory and return a pointer to it.
 *
 * Fails out gracefully if memory cannot be allocated.
 */

void *ADL__realloc(void *p, size_t bytes) {
    void *ptr = realloc(p, bytes);
    if (ptr == NULL) {
	ADL_RaiseException(ADLOutOfMemory, "Memory allocation of %ld bytes failed", bytes);
	return(NULL);
    } else {
	return ptr;
    }
}

void ADL__free(void *p) {
    free(p);
}

char *ADL__new_string(char *s) {
  char *str = ADL__malloc(strlen(s) +1);
  if (str != NULL) {
    strcpy(str, s);
  }
  return str;
}

// ***********************************************************************
// History Log:
// $Log: ADLMemory.c,v $
// Revision 1.3  1998/07/21 13:35:41  mallet
// Added new function ADL__new_string().
//
// Revision 1.2  1998/05/11 12:54:09  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.1  1998/04/16 12:09:11  shane
// First checkin of complete set.
//
//
// ***********************************************************************
