//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/c/ADLPrint.c,v $
// $Revision: 1.6 $
// $Date: 1999/03/17 16:32:16 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment - Routines to output values for
//					base types.
// NAME:	ADLPrint.c
// 	
// HISTORY:
// 	Shane McCarron, The Open Group - 4 May 1998: Created.
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLPrint.h"
#include "ADLMemory.h"
#include "ADLResultCodes.h"
#include <float.h>

static int allocs = 0;		/* the number of used alloc pointers */
static void **blocks = NULL;	/* Array of pointers to memory blocks */
static int blockcount;		/* the number of pointers available */

#define CHUNK 5
#define NUMLEN 36

/* ADL_palloc - allocate memory for use by print functions.
 *
 * ADL_palloc keeps track of the number of allocated memory chunks in the
 * internal array "blocks". This array is extended in blocks of CHUNK as needed.
 *
 * ADL_palloc returns a pointed to the allocated block of memory.
 *
 * On failure, ADL_palloc returns NULL.
 */

void *ADL_palloc(size_t size) {
    int i;
    void *ptr;
    void **new;

    if (allocs == blockcount) {
	if (blockcount) {
	    i = blockcount + CHUNK;
	    new = ADL__realloc(blocks, i * sizeof(void *));
	    if (new) {
		blocks = new;
		blockcount = i;
	    } else {
		ADL_pfree();
		allocs = 0;
	    }
	} else {
	    new = ADL__calloc(sizeof(void *), CHUNK);
	    if (new) {
		blocks = new;
		blockcount = CHUNK;
	    }
	}
	if (!new) return(NULL);
    }
    ptr = ADL__malloc(size);
    if (ptr) {
	blocks[allocs] = ptr;
	allocs++;
    }
    return ptr;
}

/* ADL_pfree - free all memory allocated via ADL_palloc.
 */

void ADL_pfree() {
    int i = 0;
    
    for (i = 0; i < allocs; i++) {
	ADL__free(blocks[i]);
	blocks[i] = NULL;
    }

    allocs = 0;
}

/* in case implementation neglects to define ANSI C constants,
   furnish minimal definitions here.
*/

#ifndef  FLT_DIG
# define FLT_DIG 6
#endif
#ifndef  DBL_DIG
# define DBL_DIG 10
#endif
#ifndef  LDBL_DIG
# define LDBL_DIG 10
#endif

char *ADL__Print_Default(void *value) {
    return("???");
}

char *ADL__Print_dPointer (void *value) {
    char *ret;

    if (ret = ADL_palloc(NUMLEN)) {
	sprintf(ret, "%p", value);
    }
    return ret;
}

char * (*ADL_Print_Pointer) (void *value) = ADL__Print_dPointer;

static char *trueString = "true";
static char *falseString = "false";

char *ADL__Print_dBoolean (int value) {
  if (value)
    return trueString;
  else
    return falseString;
}

char * (*ADL_Print_Boolean) (int value) = ADL__Print_dBoolean;

char *ADL__Print_dInt (int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%d", value);
  }
  return ptr;
}

char * (*ADL_Print_Int) (int value) = ADL__Print_dInt;

char *ADL__Print_dShort (short int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%d", value);
  }
  return ptr;
}

char * (*ADL_Print_Short) (short int value) = ADL__Print_dShort;

char *ADL__Print_dUnsignedInt (unsigned int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%u", value);
  }
  return ptr;
}

char * (*ADL_Print_UnsignedInt) (unsigned int value) = ADL__Print_dUnsignedInt;

char *ADL__Print_dUnsignedShort (unsigned short int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%d", value);
  }
  return ptr;
}

char * (*ADL_Print_UnsignedShort) (unsigned short int value) = ADL__Print_dUnsignedShort;

char *ADL__Print_dLong (long int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%d", value);
  }
  return ptr;
}

char * (*ADL_Print_Long) (long int value) = ADL__Print_dLong;

char *ADL__Print_dUnsignedLong (unsigned long int value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      sprintf(ptr, "%d", value);
  }
  return ptr;
}

char * (*ADL_Print_UnsignedLong) (unsigned long int value) = ADL__Print_dUnsignedLong;

/* This is not for multi-byte strings */
char *ADL__Print_dCharP (const char * const value) {
    int len;
    char *ptr;
    
    len = strlen(value);
    if (ptr = ADL_palloc(len + 1)) {
	strcpy(ptr, value);
    }
    return ptr;
};

char *(*ADL_Print_CharP) (const char * const value) = ADL__Print_dCharP;

char *ADL__Print_dChar (char value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      if (isprint(value)) {
	  sprintf(ptr, "%c", value);
      } else {
	  sprintf(ptr, "'\\%03o'", value);
      }
  }
  return ptr;
}

char * (*ADL_Print_Char) (char value) = ADL__Print_dChar;

char *ADL__Print_dSignedChar (signed char value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      if (isprint(value)) {
	  sprintf(ptr, "%c", value);
      } else {
	  sprintf(ptr, "'\\%03o'", value);
      }
  }
  return ptr;
}

char * (*ADL_Print_SignedChar) (signed char value) = ADL__Print_dSignedChar;


char *ADL__Print_dUnsignedChar (unsigned char value) {
  char *ptr;

  if (ptr = ADL_palloc(NUMLEN)) {
      if (isprint(value)) {
	  sprintf(ptr, "%c", value);
      } else {
	  sprintf(ptr, "'\\%03o'", value);
      }
  }
  return ptr;
}

char * (*ADL_Print_UnsignedChar) (unsigned char value) = ADL__Print_dUnsignedChar;

char *ADL__Print_dFloat(float value) {
  char *ptr;

  if (ptr = ADL_palloc(512)) {
      sprintf(ptr, "%.*g", FLT_DIG, value);
  }
  return ptr;
}

char * (*ADL_Print_Float) (float value) = ADL__Print_dFloat;

char *ADL__Print_dDouble(double value) {
  char *ptr;

  if (ptr = ADL_palloc(512)) {
      sprintf(ptr, "%.*g", DBL_DIG, value);
  }
  return ptr;
}

char * (*ADL_Print_Double) (double value) = ADL__Print_dDouble;

char *ADL__Print_dLongDouble(long double value) {
  char *ptr;

  if (ptr = ADL_palloc(512)) {
      sprintf(ptr, "%.*g", LDBL_DIG, value);
  }
  return ptr;
}

char * (*ADL_Print_LongDouble) (long double value) = ADL__Print_dLongDouble;

// ***********************************************************************
// History Log:
// $Log: ADLPrint.c,v $
// Revision 1.6  1999/03/17 16:32:16  mallet
// Bug fixed pr/160: renamed ADL__Print_dUnsigned and ADL_Print_Unsigned
// as ADL__Print_dUnsignedInt and ADL_Print_UnsignedInt.
// Bug fixed for ADL__Print_dPointer: removed incorrect derefence, the
// value is already the pointer.
//
// Revision 1.5  1998/07/21 13:33:59  mallet
// Modified ADL__Print_dPointer function to not include "0x": it's already there.
//
// Revision 1.4  1998/07/02 14:17:06  bareau
// Modified some function names (suppressed final "Int").
//
// Revision 1.3  1998/05/11 12:54:10  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.2  1998/05/08 20:21:07  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.1  1998/05/04 21:01:14  shane
// Initial version.
//
//
//
// ***********************************************************************
