// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/c/ADLTest.h,v $
// $Revision: 1.6 $
// $Date: 1998/05/28 12:32:52 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	
// NAME:	ADLTest.h
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - Oct 21, 1997
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLTEST_H
#define ADLTEST_H 1

#include "ADLDataSet.h"
#include "ADLParameterSet.h"
#include "ADLPrint.h"
#include "ADLResultCodes.h"
#include "ADLMemory.h"

// pointer to the set of parameters for the function under test.
// Contents are initialized by the generated tet_startup function.

extern ADLParameterSet *ADL_params;

// boolean indicator to indicate whether an ic has to be skipped or not
// it is set/reset according to what factories return

extern int ADL_skipAssertions;

// boolean indicator of whether reporting is turned on or not

extern int ADL_finalJournalReporting;

// boolean indicator of level of detail reporting
extern int ADL_debugMode;

// this is the adlt generated number of instances
int ADL_getNumInstances();

// declaration of an external identifier to the startup routine. This must be
// defined in the test directive .c file as the name of the "constructor"
extern void ADL_initialize();

// ADL_startup is a NULL pointer to a user-provided startup routine. This can
// be used to, for example, initialize the pointers to the custom print
// functions.
extern void (*ADL_startup)();

// Public interfaces for various routines

void ADL_infoline(char* line);
#define ADL_printf tet_printf
char* ADL_getvar(char* name);
void ADL_setblock();
void ADL_result(int res);
void ADL_assertionStart(char *assertion);
void ADL_assertionResult(int res);
void ADL_factoryResult(int res, char* msg, ...);
int tdd_result(int res, char* msg, ...);
int tdd_assert(char * assertText, int assertRes);
int tdd_skip();
int tdd_end_case(char *msg);
int tdd_end_test();

#endif // ADLTEST_H

/************************************************************************
 * History Log:
 * $Log: ADLTest.h,v $
 * Revision 1.6  1998/05/28 12:32:52  shane
 * Changed ADL_initialize to a declaration of an external void function.
 *
 * Revision 1.5  1998/05/08 20:21:08  shane
 * Various modifications, including:
 *
 * Changed to print certain information only when debugMode is true.
 * Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
 * Fixed how pointers are used (man am I stupid).
 *
 * Revision 1.4  1998/05/04 21:00:42  shane
 * Added stuff for ADLPrint.
 *
 * Revision 1.3  1998/04/30 17:01:31  shane
 * Fixed a few inconsistencies.
 *
 * Revision 1.2  1998/04/16 12:09:14  shane
 * First checkin of complete set.
 *
 * Revision 1.1  1997/10/22 13:52:55  shane
 * Initial versions.
 *
 *
 ************************************************************************/
