//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/cpp/ADLConcatenationSet.cpp,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 10:01:02 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Test directive and factory 
//                                        parameter set class
// NAME:	ADLConcatenationSet.cpp
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLConcatenationSet.hh"
#include "ADLString.hh"

void ADLConcatenationSet::addElement(ADLDataSet *e) {
  // alloc a new element set with one more element
  ADLDataSet **newSet = new ADLDataSet *[numElements + 1];

  if (numElements > 0) {
    // copy elements of old set into new one
    for (int i = 0; i < numElements; i++) {
      newSet[i]  = elementSet[i];
    }
    // now free old set
    delete elementSet;
  }

  // now set last element and set element set to newly allocated one
  newSet[numElements] = e;
  elementSet = newSet;

  // finally increase number of elements and values
  numElements += 1;
  numValues += e->getNumValues();
}

void *ADLConcatenationSet::provide(int index) throw (ADLRuntimeException) {
  // check instance number
  checkBounds(index);

  if (locateDataSet(index)) {
    ADLDataSet *d = elementSet[providedSet];
    return d->provide(providedIndex);
  } else {
    // we should normally never execute this code
    return NULL;
  }
}

bool ADLConcatenationSet::relinquish(int index) throw (ADLRuntimeException) {
  // check instance number
  checkBounds(index);

  if ((providedSet != -1) && (providedIndex != -1)) {
    ADLDataSet *d = elementSet[providedSet];
    return d->relinquish(providedIndex);
  } else {
    return true;
  }
}

void ADLConcatenationSet::checkBounds(int instance) throw (ADLRuntimeException) {
  // check to see if there are any elements
  if (numElements == 0) {
    throw ADLRuntimeException(ADLRuntime,
			      "ADLConcatenationSet: numElements == 0");
  }
  
  // check to see if the instance request is in bounds.
  if (instance < 0 || instance >= numValues) {
    throw ADLRuntimeException(ADLRuntime,
			      adlstr.reset()
			      << "ADLConcatenationSet: index out of range ("
			      << instance << "/" << numValues << ")");
  }
}

// the instance request should be starting at 0
bool ADLConcatenationSet::locateDataSet (int instance) {
    
  // reset current cache
  providedSet = -1;
  providedIndex = -1;
  
  // perform magic computation
  for (int j = 0; j < numElements; j++) {
    ADLDataSet *d = elementSet[j];
    int num = instance - d->getNumValues();
    
    if (num < 0 && instance >= 0) {
      providedSet = j;
      providedIndex = instance;
      return true;
    } else {
      instance = num;
    }
  }
  
  // alas, we've got there. So we've not found an appropriate set
  return false;
}

// ***********************************************************************
// History Log:
// $Log: ADLConcatenationSet.cpp,v $
// Revision 1.5  1998/02/27 10:01:02  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/19 09:54:39  mallet
// Updated checkBounds according to its new signature and semantics.
// Updated use of checkBounds in provide and relinquish according to its
// new signature.
//
// Revision 1.3  1998/02/12 16:04:25  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 16:42:44  mallet
// Ported to Sun's CC (C++ compiler): fixed use of new for array of pointers.
//
// Revision 1.1  1997/11/14 10:13:28  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
