//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/cpp/ADLFactorySet.hh,v $
// $Revision: 1.6 $
// $Date: 1998/02/27 10:01:04 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Factory set class
// NAME:	ADLFactorySet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Nov  2, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLFactorySet_hh
#define ADLFactorySet_hh

#include "ADLDataSet.hh"
#include "ADLParameterSet.hh"
#include "ADLTest.hh"
#include "ADLRuntimeException.hh"

/*
 *  class ADLFactorySet
 *
 *  The ADLFactorySet represents a TDD factory which is used to build
 *  other TDD datasets. Its number of values is the product of the number
    of values of its parameters.
 *
 */
class ADLFactorySet : public ADLDataSet {
public:
  ADLFactorySet(ADLTest *tc, char *fname = "Anonymous");

  void addParameter(ADLDataSet *e);

  int getNumParameters(void);

  void *provide(int index) throw (ADLRuntimeException);

  bool relinquish(int index) throw (ADLRuntimeException);

protected:
  char *factoryName;

  virtual void *factory_provide(void **params) = 0;

  //  virtual bool factory_relinquish(void **params, void *obj) = 0;
  bool factory_relinquish(void** params, void* obj);

  void infoline(char *line) { testContext->infoline(line); }

  bool tdd_end_case(char *msg = NULL) { return testContext->tdd_end_case(msg); }

  bool tdd_skip(void) { return testContext->tdd_skip(); }

  bool tdd_end_test(void) { return testContext->tdd_end_test(); }

  bool tdd_result(int code, char *msg = NULL) {
    return testContext->tdd_result(code, msg); }

private:
  ADLParameterSet *params; // factory parameters

  void **factoryParameters; // cache of last parameters to factory
  void *providedObject; // cache of last provided object

};
#endif // ADLFactorySet_hh

// ***********************************************************************
// History Log:
// $Log: ADLFactorySet.hh,v $
// Revision 1.6  1998/02/27 10:01:04  mallet
// Updated copyright notice.
//
// Revision 1.5  1998/02/12 16:04:27  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.4  1998/02/11 18:12:10  mallet
// Added specific processing for ADLRuntimeException in provide and
// relinquish to allow tdd_end_case and tdd_end_test to act as expected:
// ADLRuntimeException is propagated through.
//
// Revision 1.3  1998/01/29 12:53:46  bareau
// Update of the C++ runtime.
// In ADLFactorySet, factory_relinquish is no more virtual
// but has a default behavior "return true".
// In ADLLiteralSet, added a default constructor with one parameter
// ADLTest *, and a method "void setValues(ADLLiteralSet* copySet)"
// to create an ADLLiteralSet from an other one.
//
// Revision 1.2  1997/11/21 16:51:03  mallet
// Replaced string by char* to become more portable.
// Changed scope of tdd_* methods: now protected.
// Added an infoline method so that it can be used in the body of a factory.
//
// Revision 1.1  1997/11/14 10:13:29  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
