//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/cpp/ADLParameterSet.hh,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 10:01:05 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - Test directive and factory 
//                                        parameter set class
// NAME:	ADLParameterSet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLParameterSet_hh
#define ADLParameterSet_hh

#include "ADLDataSet.hh"
#include "ADLRuntimeException.hh"
#include <stdlib.h>

/*
 *  class ADLParameterSet
 *
 *  The ADLParameterSet implements a list of dataset parameters for
 *  test directive and factory parameters.
 *
 *  @see ADLDataSet
 *
 */
class ADLParameterSet {
public:
  ADLParameterSet(void);

  int getNumValues(void) { return numValues; }

  int getNumElements(void) { return numElements; }

  void addElement(ADLDataSet *e);

  void **provide(int index) throw (ADLRuntimeException);

  bool relinquish(int index) throw (ADLRuntimeException);

private:
  ADLDataSet **elementSet; // the list of ADLDataSet
  int numElements; // number of parameters
  int numValues; // number of values (when there is no parameter in
  // the list, it can return a single value)

  int *elementIndexes; // local cache of provided indexes

  int *computeArrayofMaxInstances(void);

  int *computeIndexes(int instance) throw (ADLRuntimeException);

  void checkBounds(int index) throw (ADLRuntimeException);
};

#endif // ADLParameterSet_hh

// ***********************************************************************
// History Log:
// $Log: ADLParameterSet.hh,v $
// Revision 1.5  1998/02/27 10:01:05  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/19 09:50:38  mallet
// Changed signature and semantics of checkBounds to throw an
// ADLRuntimeException instead of returning a boolean.
// Changed signature of computeIndexes, provide and relinquish to throw an
// ADLRuntimeException when checkBounds fails instead of returning NULL,
// which could be gently ignored.
// Changed usage of checkBounds in computeIndexes, provide and relinquish
// according to its new signature.
//
// Revision 1.3  1998/02/12 16:04:28  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 16:59:40  mallet
// Fixed prototypes for methods with no parameters (require a void).
//
// Revision 1.1  1997/11/14 10:13:31  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
