//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togcvs/adl2/src/runtime/cpp/ADLTest.hh,v $
// $Revision: 1.8 $
// $Date: 1998/05/12 17:03:19 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	
// NAME:	ADLTest.hh
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - Oct 21, 1997
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLTest_hh
#define ADLTest_hh

#include "ADLResultCodes.h"
#include "ADLParameterSet.hh"
#include "ADLDataSet.hh"
#include "ADLRuntimeException.hh"

class ADLTest {
private:

  // holds the parameter list for the test
  ADLParameterSet *params;

public:
  ADLTest(void);

  // boolean indicator to indicate whether an ic has to be skipped or not
  // it is set/reset according to what factories return
  bool skipAssertions;

  // indicates whether certain output should be placed in the journal
  bool debugMode;

  void addParameter(ADLDataSet *p);
  int getNumInstances(void);
  void runTestInstance(int testIndex);
  virtual void runInstance(void **parameters) = 0;
  void infoline(char *line);
  char* getvar(char* name);
  void setblock(void);
  void result(int res);
  void assertionStart(char* assertion);
  void assertionResult(int res);
  void factoryResult(int res, char* msg);
  bool tdd_result(int res, char* msg);
  bool tdd_assert(char* assertText, bool assertRes);
  bool tdd_skip(void);
  bool tdd_end_case(char* msg) throw (ADLRuntimeException);
  bool tdd_end_test(void) throw (ADLRuntimeException);
};

extern ADLTest *ADL_theTest;


#endif // ADLTest_hh

// ***********************************************************************
// History Log:
// $Log: ADLTest.hh,v $
// Revision 1.8  1998/05/12 17:03:19  shane
// Added handling for debugMode where appropriate to match C and Java runtimes.
// Added debugMode to ADLTest class as a public bool.
//
// Revision 1.7  1998/02/27 10:01:07  mallet
// Updated copyright notice.
//
// Revision 1.6  1998/02/12 16:04:30  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/02/11 18:14:05  mallet
// Updated runTestInstance so that exceptions raised during provide and
// relinquish are also caught.
// Changed tdd_skip to set result to ADL_SKIPPED: was ADL_UNINITIATED.
//
// Revision 1.4  1997/11/24 16:06:43  mallet
// Moved include of ADLResultCodes in header file to simplify generated
// code which now doesn't need to include ADLResultCodes directly.
// Fixed tdd_assert method to report on the text of the assertion to
// check and to use assertionResult instead of result.
//
// Revision 1.3  1997/11/21 17:05:08  mallet
// Fixed prototypes for methods with no arguments: void required.
//
// Revision 1.2  1997/11/14 10:24:06  mallet
// Updated with new classes added.
// Changed return type of tdd_* methods: now use bool.
// Changed declaration of ADL_theTest to be ADLTest* instead of ADLTest&.
//
// Revision 1.1  1997/10/22 11:27:59  shane
// Initial version.
//
//
// ***********************************************************************
